@@ -15,6 +15,7 @@
 #include "global.h"
 #include "iniParser.h"
 #include "launchoptions.h"
+#include "prelaunchsplashhelper.h"
 #include "propertiesForwarder.h"
 #include <DConfig>
 #include <QList>
@@ -34,6 +35,7 @@
 #include <qtmetamacros.h>
 #include <utility>
 #include <wordexp.h>
+#include <QLoggingCategory>
 
 using namespace Qt::Literals::StringLiterals;
 
@@ -281,6 +283,11 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
         execStr = realExec;
     }
 
+    auto workDir = m_entry->value(DesktopFileEntryKey, "Path");
+    if (workDir && !optionsMap.contains(setWorkingPathLaunchOption::key())) {
+        optionsMap.insert(setWorkingPathLaunchOption::key(), workDir->toString());
+    }
+
     while (execStr.isEmpty() and !action.isEmpty() and !supportedActions.isEmpty()) {  // break trick
         if (auto index = supportedActions.indexOf(action); index == -1) {
             qWarning() << "can't find " << action << " in supported actions List. application will use default action to launch.";
@@ -319,6 +326,20 @@ ApplicationService::Launch(const QString &action, const QStringList &fields, con
         }
     }
 
+    // Notify the compositor to show a splash screen if in a Wayland session.
+    if (auto *am = parent()) {
+        if (auto *helper = am->splashHelper()) {
+            const auto iconVar = findEntryValue(DesktopFileEntryKey, "Icon", EntryValueType::IconString);
+            const QString iconName = iconVar.isNull() ? QString{} : iconVar.toString();
+            qCInfo(amPrelaunchSplash) << "Show prelaunch splash request" << id() << "icon" << iconName;
+            helper->show(id(), iconName);
+        } else {
+            qCInfo(amPrelaunchSplash) << "Skip prelaunch splash (no helper instance)" << id();
+        }
+    } else {
+        qCWarning(amPrelaunchSplash) << "Skip prelaunch splash (no parent ApplicationManager1Service)" << id();
+    }
+
     optionsMap.remove("_hooks");  // this is internal property, user shouldn't pass it to Application Manager
     if (const auto &hooks = parent()->applicationHooks(); !hooks.isEmpty()) {
         optionsMap.insert("_hooks", hooks);
